package de.towerdefence.server.match.confirmation;

import de.towerdefence.server.player.Player;
import lombok.Getter;

import java.time.Instant;
import java.util.Optional;
import java.util.UUID;

@Getter
public class UnconfirmedMatch {
    private final String matchId;
    private final long created;
    public static final long TTL = 30 * 1000;
    private final Player player1;
    private final Player player2;
    private final ConfirmationCallbacks player1Callbacks;
    private final ConfirmationCallbacks player2Callbacks;

    public UnconfirmedMatch(
        Player player1,
        Player player2,
        ConfirmationCallbacks player1Callbacks,
        ConfirmationCallbacks player2Callbacks) {
        this.player1 = player1;
        this.player2 = player2;
        this.player1Callbacks = player1Callbacks;
        this.player2Callbacks = player2Callbacks;
        this.created = Instant.now().toEpochMilli();
        this.matchId = UUID.randomUUID().toString();
    }

    private PlayerMatchConfirmState player1State = PlayerMatchConfirmState.UNKNOWN;
    private PlayerMatchConfirmState player2State = PlayerMatchConfirmState.UNKNOWN;

    public Optional<UnconfirmedMatchState> setPlayerConfirmState(Player player, PlayerMatchConfirmState state) {
        Optional<MatchPlayer> matchPlayer = getMatchPlayer(player);
        if (matchPlayer.isEmpty()) {
            return Optional.empty();
        }

        switch (matchPlayer.get()) {
        case ONE -> player1State = state;
        case TWO -> player2State = state;
        }

        boolean timedOut = Instant.now().toEpochMilli() > created + TTL;
        if (
            timedOut
                || player1State == PlayerMatchConfirmState.ABORTED
                || player2State == PlayerMatchConfirmState.ABORTED
        ) {
            return Optional.of(UnconfirmedMatchState.ABORTED);
        }
        if (player1State == PlayerMatchConfirmState.UNKNOWN || player2State == PlayerMatchConfirmState.UNKNOWN) {
            return Optional.of(UnconfirmedMatchState.WAITING);
        }
        return Optional.of(UnconfirmedMatchState.CONFIRMED);
    }

    public Optional<PlayerMatchConfirmState> getPlayerState(Player player) {
        Optional<MatchPlayer> matchPlayer = getMatchPlayer(player);
        if (matchPlayer.isEmpty()) {
            return Optional.empty();
        }

        return switch (matchPlayer.get()) {
            case ONE -> Optional.of(player1State);
            case TWO -> Optional.of(player2State);
        };
    }

    private enum MatchPlayer {
        ONE,
        TWO
    }

    private Optional<MatchPlayer> getMatchPlayer(Player player) {
        boolean isPlayerOne = player.equals(player1);
        boolean isPlayerTwo = player.equals(player2);
        if (!isPlayerOne && !isPlayerTwo) {
            return Optional.empty();
        }

        if (isPlayerOne) {
            return Optional.of(MatchPlayer.ONE);
        }
        return Optional.of(MatchPlayer.TWO);
    }
}
