package de.towerdefence.server.server.channels.match.enemy;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;

import de.towerdefence.server.game.Enemy;
import de.towerdefence.server.server.JsonMessage;
import lombok.AllArgsConstructor;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

@AllArgsConstructor
public class EnemyUpdateMessage extends JsonMessage {

    private final Enemy[] newEnemies;
    private final Enemy[] changedEnemies;

    @Override
    protected String getMessageId() {
        return "EnemyUpdate";
    }

    @Override
    protected Map<String, JsonNode> getData(JsonNodeFactory factory) {

        List<JsonNode> newArray = Arrays
                .stream(newEnemies)
                .map((Enemy enemy) -> {
                    ObjectNode o = factory.objectNode();
                    o.set("id", factory.numberNode(enemy.getId()));
                    o.set("x", factory.numberNode(enemy.getPos().x));
                    o.set("y", factory.numberNode(enemy.getPos().y));
                    return o;
                })
                .map(JsonNode.class::cast)
                .toList();

        List<JsonNode> changedArray = Arrays
                .stream(changedEnemies)
                .map((Enemy enemy) -> {
                    ObjectNode o = factory.objectNode();
                    o.set("id", factory.numberNode(enemy.getId()));
                    o.set("x", factory.numberNode(enemy.getPos().x));
                    o.set("y", factory.numberNode(enemy.getPos().y));
                    return o;
                })
                .map(JsonNode.class::cast)
                .toList();

        return Map.of(
            "new", new ArrayNode(factory, newArray),
            "changed", new ArrayNode(factory, changedArray)
        );
    }
}
